/*#######################################################################################
ATmega8535,16,32,64 and 128  ADC Functions  

Copyright (C) 2005 Branko Golubovic

branko.golubovic@gmx.at
#######################################################################################*/

#include "main.h"
#include "adc.h"
//############################################################################

/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}


/************************************************************************* 
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delayFourCycles( ( ( 1*(SYSCLK/4000) )*us)/1000 )

//############################################################################
// Routine liest Ein Analog eingang
signed int read_adc(unsigned char adcinput) 
//############################################################################
{
#if defined (USE_DIFF_INPUTS)
static unsigned char adcinput_old;
signed int SADCdata;
unsigned int UADCdata;
#else
unsigned int ADCdata;
#endif

#if defined (USE_DIFF_INPUTS)
ADMUX=adcinput|ADC_VREF_TYPE|(1<< ADLAR);	// ADC_MUX Upload ,AD Left Adjust Result
#else
ADMUX=adcinput|ADC_VREF_TYPE;
#endif

#if defined (USE_DIFF_INPUTS)
// Warte 125s beim Wekseln von Single Ended auf Diff. Inputs oder zwischen Diff. Inputs
if((adcinput>0x07) && (adcinput<0x01E) && (adcinput!=adcinput_old)) delay(125);	// Warte  125S  
#endif

// Starte AD Konversion
ADCSRA|=1<<ADSC;


while ((ADCSRA & (1<<ADIF))==0);// Warte  bis AD Konversion fertig ist
ADCSRA|=(1<<ADIF);	// Lsche ADC Interrupt Flag
#if defined (USE_DIFF_INPUTS) 
adcinput_old=adcinput;	// Lezte ADC Input Speichern
#endif

#if defined (USE_DIFF_INPUTS)
if((adcinput>0x07) && (adcinput<0x01E))
	{
	SADCdata=ADCW;
	return SADCdata/64 ;	// Ergebnis von-512 bis 511,Differential inputs
	}
else	
	{
	UADCdata=ADCW	;	 	
	return UADCdata/64;	// Ergebnis von  0 bis 1023
	}
#else
	return ADCdata;			// Ergebnis von  0 bis 1023
#endif

}
 
 
//############################################################################
// Routine liest Aufsummiert Ein Analog eingang
signed int read_adc_summe(unsigned char summe, unsigned char input)
//############################################################################

{	
	unsigned char s;
	signed int ADC_mittelwert=0;

	if (summe>64) s=64;
	else
	s=summe;
	
	
	for(char i=0;i<s;i++)
	{
		ADC_mittelwert+=read_adc(input);
	}
	return ADC_mittelwert/s;
}


void ADC_init(void)
{
// ADC initialization
// ADC Clock frequency:SYSCLK/ADC_PRESCALER
// ADC Voltage Reference: ADC_VREF_TYPE

#if defined (USE_DIFF_INPUTS)
ADMUX=ADC_VREF_TYPE|(1<<ADLAR);
#else
ADMUX=ADC_VREF_TYPE;
#endif

ADCSRA=(1<<ADEN)|ADC_PRESCALER;

}



