/*#######################################################################################
AVR ATmega32 ADC TEST PROGRAMM

Das Programm tested Schwankungen bei angeschlossene konstante Spannung
zwischen ADC0 und ADC1 als Single Ended Eingang,und Differential
Eingang bei 1,10 und 200-fache Verstrkung.

Achtung!!! Angeschlossene Spannung darf nicht AVCC bersteigen
und Spannungdifferentz Zwischen ADC0 und ADC1 Eingang darf nicht
grser als 20mV sein  wegen 200-fche Verstrkung!!!
         
      |<<  Index_Offset >>|<-ARRAY_SIZE/2
										 Y
										 ^
      I              |    i        
 		 III             |   iii  
 		IIIII            |  iiiii
    IIIII            |  iiiii
 	 IIIIIII           | iiiiiii 
-------------------------------------------->X
      |<-ADCdata     |0<<Index>>|<-ARRAY_SIZE

X Achse -> ADC Data
Y Achse -> Hufigkeit
I -> Richtige Hufigkeit
i -> Verschobene Hufigkeit=I+Index_offset

Programm misst 1000 mall gewelte Analog Eingang, und bei jede
abeschlossene ADC Konversion Aktualisiert "ADC_data_array[] "Feld.

Messergebnis wird ber Serielle Schnittstelle (9600,8,N,1)
auf Terminall Programm bertragen.


Copyright (C) 2005 Branko Golubovic
branko.golubovic@gmx.at
*/

#include "main.h"

//############################################################################

#define ARRAY_SIZE	16



 const unsigned char VREF_message[3][35]PROGMEM={
 {"#  VREF=External Vref Pin\n"},
 {"#  VREF=AVCC\n"},
 {"#  VREF=Interne 2,56V Referentz \n"}};
 
 
  
  const unsigned char ADC_input_message[10][50]PROGMEM={
 {"#### 1.22V Bandgap Reference,Single ended ####\n"},// MUX=0x1E
 {"#############  ADC0 Single ended   ###########\n"},// MUX=0
 {"# ADC1-ADC1(Offset Test), Diff Input, Gain=1 #\n"},// MUX=0x11
 {"####### ADC0-ADC1, Diff Input, Gain=1 ########\n"},// MUX=0x10
 {"# ADC0-ADC0(Offset Test), Diff Input,Gain=10 #\n"},// MUX=0x08
 {"####### ADC1-ADC0, Diff Input, Gain=10 #######\n"},// MUX=0x09
 {"# ADC0-ADC0(Offset Test),Diff Input,Gain=200 #\n"},// MUX=0x0A
 {"####### ADC1-ADC0, Diff Input, Gain=200 ######\n"},// MUX=0x0B
 {"# ADC0-ADC0(Offset Test),Summe16   ,Gain=200 #\n"},// MUX=0x0A
 {"####### ADC1-ADC0, Summe16 , Gain=200 ########\n"}};// MUX=0x0B

 const unsigned char ADC_MUX[10]={0x1E,0,0x11,0x10,0x08,0x09,0x0A,0x0B,0x0A,0x0B};


//Hauptprogramm
int main (void)

//############################################################################
{	
unsigned int ADC_out_of_range;
signed int ADC_data_array[ARRAY_SIZE];
signed int ADC_data,ADC_mittelwert;
	IO_init();
	UART_Init(); 
	ADC_init();
	
	//<Show Header>
	printf_P(PSTR("###########  PROGRAMM ADC_TEST ##############\n"));
	switch(ADC_VREF_TYPE)
	{
		case 0x00:	printf_P(VREF_message[0]);
								break;
		case 0x40:	printf_P(VREF_message[1]);
								break;
		case 0xC0:	printf_P(VREF_message[2]);
								break;
	}
	printf_P(PSTR("#  ADC clock=%-dKHz\n"),SYSCLK_KHZ/(1<<ADC_PRESCALER));
	printf_P(PSTR("#############################################\n\n"));
	//</Show Header>
	
	wait(10000000);	// Warte paar Sekunden bis sich Spannung stabilisiert
	
	for(char i=0;i<10;i++)	// Zehn mall wird gemessen
	{
	signed int ARRAY_index,Index_offset;
	
		
		for(char j=0;j<ARRAY_SIZE;j++)	
		{
			ADC_data_array[j]=0;	// ADC_data_array[] lschen
		}		
		ADC_out_of_range=0;
		
		ADC_mittelwert=read_adc_summe(16,ADC_MUX[i]);		//Mittelwert von 16 Messungen		
					
		Index_offset=(ARRAY_SIZE/2)-ADC_mittelwert;	//Differentz zwischen ADC_mittelwert und ARRAY_SIZE/2 Umrechnen
		
		printf_P(PSTR("ADCin%-d  Mittelwert= %-d\n"),ADC_MUX[i],ADC_mittelwert);	// Gewhlte ADCin und ADC mittelwert zeigen
		
			for(int k=0;k<1000;k++)	//
			{
				if(i<8)
				ADC_data=read_adc(ADC_MUX[i]);	// Messresultat ohne aufsummierung
				else
				ADC_data=read_adc_summe(16,ADC_MUX[i]); //Aufsummierung von 16 messungen fr 200-fache verstrkung
				
				ARRAY_index=(Index_offset+ADC_data);	//ADCdata ins (0 bis ARRAY_SIZE-1) Grnzen Verschiben
				if((ARRAY_index<0)||(ARRAY_index>(ARRAY_SIZE-1)))// berprfung von Array_index Grenze(0 bis ARRAY_SIZE-1)
				{
				ADC_out_of_range++;	// Leider hat nich gepast!,Schwankungen sind zu gross
				}
				else
				{
				ADC_data_array[ARRAY_index]++;// Bingo!,Zutrefende feld incrementieren
				}
			}
			printf_P(ADC_input_message[i]);
			printf_P(PSTR("ADCdata\tFreq.\t0%%        20%%       40%%       60%%       80%%       100%%  \n"));
			printf_P(PSTR("\t\t\I         I         I         I         I         I\n"));
			
				for(char l=0;l<ARRAY_SIZE;l++) 
				{
				unsigned int Sharp;
				//Richtige ADCdata rekonstruieren und zeigen zusahmen mit gemessene Hufigkeit
				printf_P(PSTR("%-d\t%-d\t"),l-Index_offset,ADC_data_array[l]);
				
				Sharp=(ADC_data_array[l]/20); 
					while(Sharp)
					{
					printf("#"); Sharp--;// Hufigkeit als "Grafik" darstellen
					}
					printf("\n");
				}
			printf_P(PSTR("Out of Range=%-d\n\n\n"),ADC_out_of_range);
	}
	
	printf_P(PSTR("TEST END!\n"));
	while (1)
		{ 
		
		}
return (1);
}// Main End!

	
	
	